﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using ThinkGeo.GpstrackingServer.Geteways.GatewaySdk;

namespace ThinkGeo.GpstrackingServer.Geteways.NMEAWebListener
{
    public partial class _Default : System.Web.UI.Page
    {
        // Sample Message: "351277001110161$GPRMC,155726,A,3857.2284,N,09516.9031,W,000.0,208.3,170106,002.4,E,A*18,AUTO";
        // Here is your Api key
        private const string ApiKey = "<Your Api Key>";
        // Here is your gateway web service
        private const string GatewayWebServiceUri = "https://gpsgateway.thinkgeo.com";  
        private GatewayManager mGatewayManager;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                if (Request.QueryString["message"] != null)
                {
                    if (Request.QueryString["message"].ToString() != "")
                    {
                        mGatewayManager = new GatewayManager();
                        ProcessRecvMsgs(Request.QueryString["message"].ToString());
                    }
                }
            }
        }

        /// <summary>
        /// Process the message
        /// </summary>
        /// <param name="receiveString"></param>
        private void ProcessRecvMsgs(string receiveString)
        {
            string DeviceId = "";
            try
            {
                DeviceId = receiveString.Substring(0, 15);
                DateTime UtcDateTime = DateTime.MinValue;
                int AltitudeInMeters = 0;
                string RawDebugMessage = receiveString;
                receiveString = receiveString.Replace(DeviceId, "");
                receiveString = receiveString.Replace("$", "");
                string[] Parts = receiveString.Split(Convert.ToChar(","));
                string ValidRecord = Parts[2].ToString();
                int GpsAccuracyInMeters = -1;
                if (ValidRecord == "A")
                {
                   GpsAccuracyInMeters = 1;
                }

                string TimeString = Parts[1].ToString();
                int Hour, Minute, Second;
                Hour = Convert.ToInt32(TimeString.Substring(0, 2));
                Minute = Convert.ToInt32(TimeString.Substring(2, 2));
                Second = Convert.ToInt32(TimeString.Substring(4, 2));
                string DateString = Parts[9].ToString();

                int Day, Month, Year;
                Day = Convert.ToInt32(DateString.Substring(0, 2));
                Month = Convert.ToInt32(DateString.Substring(2, 2));
                Year = Convert.ToInt32(DateString.Substring(4, 2)) + 2000;

                UtcDateTime = new DateTime(Year, Month, Day, Hour, Minute, Second);
                string LatitudeString = Parts[3].ToString();
                string LatitudeDirection = Parts[4].ToString();
                decimal Latitude = DMToDecimalDegrees(Convert.ToDouble(LatitudeString.Substring(0, 2)), Convert.ToDouble(LatitudeString.Substring(2, 7)), LatitudeDirection);

                string LongitudeString = Parts[5].ToString();
                string LongitudeDirection = Parts[6].ToString();
                decimal Longitude = DMToDecimalDegrees(Convert.ToDouble(LongitudeString.Substring(0, 3)), Convert.ToDouble(LongitudeString.Substring(3, 7)), LongitudeDirection);

                int Speed = Convert.ToInt32(Convert.ToDouble(Parts[7].Substring(0, 3)) * 1.852);
                int SpeedInKph = (int)(Speed * 1.85200); 
                int Heading = Convert.ToInt16(Math.Floor(Convert.ToDouble("0" + Parts[8])));

                DeviceMessage deviceMessage = new DeviceMessage(DeviceId,
                    UtcDateTime, Longitude, Latitude, GpsAccuracyInMeters, AltitudeInMeters, SpeedInKph, Heading);

                // Here to get the custom data from the recive string.
                // Eg. CustomDataValue = GetDistance(receiveString);
                string CustomDataValue = "100";
                deviceMessage.CustomData.Add(new CustomData("Distance", CustomDataValue, MimeType.Text));

                string Status = Parts[13].ToString();
                if (Status.ToUpper().StartsWith("SOS"))
                {
                    deviceMessage.Alerts.Add("Panic");
                }
                if (Status.ToUpper().StartsWith("LP"))
                {
                    deviceMessage.Alerts.Add("LowBattery");
                }

                PostResult myResult = mGatewayManager.PostDeviceMessage(deviceMessage);
                Response.Write(string.Format("The post result code:{0}, message:{1}", myResult.Code.ToString(), myResult.ErrorMessage));

                if (myResult.Code == ResultCode.Error)
                {
                    mGatewayManager.LogDeviceError(DeviceId, "NMEAWebListener", myResult.ErrorMessage);
                }

            }
            catch (Exception Ex)
            {
                 Response.Write(Ex.ToString());
                 mGatewayManager.LogDeviceError(DeviceId, "NMEAWebListener", Ex.ToString());
            }
        }
       
        /// <summary>
        /// Convert the "Degrees + Minutes + Direction" format to Decimal Degrees
        /// </summary>
        /// <param name="Degrees"></param>
        /// <param name="Minutes"></param>
        /// <param name="Direction"></param>
        /// <returns></returns>
        private decimal DMToDecimalDegrees(double Degrees, double Minutes, string Direction)
        {
            double DecMin;
            double DecDegrees;
            DecMin = Minutes / 60;
            if (Degrees >= 0)
            {
                DecDegrees = Degrees + DecMin;
            }
            else
            {
                DecDegrees = Degrees - DecMin;
            }
            if (Direction == "W" | Direction == "S")
            {
                DecDegrees = DecDegrees * -1;
            }
            return Convert.ToDecimal(DecDegrees);
        }
    }
}
